@@ -15,11 +15,13 @@ from typing import (
     Iterator,
     Mapping,
     Optional,
+    Sequence,
     TextIO,
     Type,
     TypeVar,
     Union,
 )
+from urllib.parse import ParseResult
 
 import pysnooper
 import yaml
@@ -51,16 +53,10 @@ else:
 
 
 from .format import PythonVersion
-from .model.pydantic import (
-    BaseModel,
-    CustomRootType,
-    DataModelField,
-    dump_resolve_reference_action,
-)
-from .model.pydantic.types import DataTypeManager
-from .parser import LiteralType
+from .model.pydantic import dump_resolve_reference_action
+from .parser import DefaultPutDict, LiteralType
 from .parser.base import Parser
-from .version import version as __version__
+from .types import StrictTypes
 
 T = TypeVar('T')
 
@@ -68,7 +64,6 @@ pysnooper.tracer.DISABLED = True
 
 DEFAULT_BASE_CLASS: str = 'pydantic.BaseModel'
 
-
 SafeLoader.yaml_constructors[
     'tag:yaml.org,2002:timestamp'
 ] = SafeLoader.yaml_constructors['tag:yaml.org,2002:str']
@@ -78,6 +73,11 @@ def load_yaml(stream: Union[str, TextIO]) -> Any:
     return yaml.load(stream, Loader=SafeLoader)
 
 
+def load_yaml_from_path(path: Path, encoding: str) -> Any:
+    with path.open(encoding=encoding) as f:
+        return load_yaml(f)
+
+
 def enable_debug_message() -> None:  # pragma: no cover
     pysnooper.tracer.DISABLED = False
 
@@ -168,7 +168,7 @@ def get_first_file(path: Path) -> Path:  # pragma: no cover
 
 
 def generate(
-    input_: Union[Path, str],
+    input_: Union[Path, str, ParseResult],
     *,
     input_filename: Optional[str] = None,
     input_file_type: InputFileType = InputFileType.Auto,
@@ -196,25 +196,36 @@ def generate(
     strict_nullable: bool = False,
     use_generic_container_types: bool = False,
     enable_faux_immutability: bool = False,
+    disable_appending_item_suffix: bool = False,
+    strict_types: Optional[Sequence[StrictTypes]] = None,
+    empty_enum_field_name: Optional[str] = None,
 ) -> None:
-    input_text: Optional[str] = None
+    remote_text_cache: DefaultPutDict[str, str] = DefaultPutDict()
+    if isinstance(input_, str):
+        input_text: Optional[str] = input_
+    elif isinstance(input_, ParseResult):
+        from .http import get_body
+
+        input_text = remote_text_cache.get_or_put(
+            input_.geturl(), default_factory=get_body
+        )
+    else:
+        input_text = None
+
     if isinstance(input_, Path) and not input_.is_absolute():
         input_ = input_.expanduser().resolve()
-
     if input_file_type == InputFileType.Auto:
         try:
             input_text_ = (
-                input_
-                if isinstance(input_, str)
-                else get_first_file(input_).read_text(encoding=encoding)
+                get_first_file(input_).read_text(encoding=encoding)
+                if isinstance(input_, Path)
+                else input_text
             )
             input_file_type = (
                 InputFileType.OpenAPI
-                if is_openapi(input_text_)
+                if is_openapi(input_text_)  # type: ignore
                 else InputFileType.JsonSchema
             )
-            if isinstance(input_, str):
-                input_text = input_text_
         except:
             raise Error('Invalid file format')
 
@@ -244,18 +255,18 @@ def generate(
                         csv_reader = csv.DictReader(csv_file)
                         return dict(zip(csv_reader.fieldnames, next(csv_reader)))  # type: ignore
 
-                    if isinstance(input_, str):
-                        import io
-
-                        obj = get_header_and_first_line(io.StringIO(input_))
-                    else:
+                    if isinstance(input_, Path):
                         with input_.open(encoding=encoding) as f:
                             obj = get_header_and_first_line(f)
+                    else:
+                        import io
+
+                        obj = get_header_and_first_line(io.StringIO(input_text))
                 else:
                     obj = load_yaml(
-                        input_
-                        if isinstance(input_, str)
-                        else input_.read_text(encoding=encoding)
+                        input_.read_text(encoding=encoding)  # type: ignore
+                        if isinstance(input_, Path)
+                        else input_text
                     )
             except:
                 raise Error('Invalid file format')
@@ -266,7 +277,7 @@ def generate(
             input_text = json.dumps(builder.to_schema())
 
     parser = parser_class(
-        source=input_text or input_,
+        source=input_ if isinstance(input_, ParseResult) else input_text or input_,
         base_class=base_class,
         custom_template_dir=custom_template_dir,
         extra_template_data=extra_template_data,
@@ -292,6 +303,10 @@ def generate(
         strict_nullable=strict_nullable,
         use_generic_container_types=use_generic_container_types,
         enable_faux_immutability=enable_faux_immutability,
+        remote_text_cache=remote_text_cache,
+        disable_appending_item_suffix=disable_appending_item_suffix,
+        strict_types=strict_types,
+        empty_enum_field_name=empty_enum_field_name,
     )
 
     with chdir(output):
@@ -299,6 +314,8 @@ def generate(
     if not input_filename:  # pragma: no cover
         if isinstance(input_, str):
             input_filename = '<stdin>'
+        elif isinstance(input_, ParseResult):
+            input_filename = input_.geturl()
         else:
             input_filename = input_.name
     if not results:
@@ -321,9 +338,9 @@ def generate(
 
     timestamp = datetime.now(timezone.utc).replace(microsecond=0).isoformat()
 
-    header = f'''\
+    header = '''\
 # generated by datamodel-codegen:
-#   filename:  {{filename}}'''
+#   filename:  {}'''
     if not disable_timestamp:
         header += f'\n#   timestamp: {timestamp}'
 
@@ -337,7 +354,7 @@ def generate(
                 path.parent.mkdir(parents=True)
             file = path.open('wt', encoding=encoding)
 
-        print(header.format(filename=filename), file=file)
+        print(header.format(filename), file=file)
         if body:
             print('', file=file)
             print(body.rstrip(), file=file)
