@@ -13,7 +13,8 @@ from collections import defaultdict
 from enum import IntEnum
 from io import TextIOBase
 from pathlib import Path
-from typing import Any, DefaultDict, Dict, Optional, Sequence, cast
+from typing import Any, DefaultDict, Dict, List, Optional, Sequence, Union, cast
+from urllib.parse import ParseResult, urlparse
 
 import argcomplete
 import black
@@ -29,6 +30,8 @@ from datamodel_code_generator import (
     generate,
 )
 from datamodel_code_generator.parser import LiteralType
+from datamodel_code_generator.reference import is_url
+from datamodel_code_generator.types import StrictTypes
 
 from .format import PythonVersion, is_supported_in_black
 
@@ -53,12 +56,16 @@ arg_parser = ArgumentParser()
 arg_parser.add_argument(
     '--input', help='Input file/directory (default: stdin)',
 )
+arg_parser.add_argument(
+    '--url', help='Input file URL. `--input` is ignore when `--url` is used',
+)
 arg_parser.add_argument(
     '--input-file-type',
     help='Input file type (default: auto)',
     choices=[i.value for i in InputFileType],
 )
 arg_parser.add_argument('--output', help='Output file (default: stdout)')
+
 arg_parser.add_argument(
     '--base-class', help='Base Class (default: pydantic.BaseModel)', type=str,
 )
@@ -80,7 +87,12 @@ arg_parser.add_argument(
     action='store_true',
     default=None,
 )
-
+arg_parser.add_argument(
+    '--disable-appending-item-suffix',
+    help='Disable appending `Item` suffix to model name in an array',
+    action='store_true',
+    default=None,
+)
 arg_parser.add_argument(
     '--allow-population-by-field-name',
     help='Allow population by field name',
@@ -116,6 +128,13 @@ arg_parser.add_argument(
     default=None,
 )
 
+arg_parser.add_argument(
+    '--strict-types',
+    help='Use strict types',
+    choices=[t.value for t in StrictTypes],
+    nargs='+',
+)
+
 arg_parser.add_argument(
     '--disable-timestamp',
     help='Disable timestamp on file headers',
@@ -168,6 +187,13 @@ arg_parser.add_argument(
     default=None,
 )
 
+arg_parser.add_argument(
+    '--empty-enum-field-name',
+    help='Set field name when enum value is empty (default:  `_`)',
+    default=None,
+)
+
+
 arg_parser.add_argument(
     '--class-name', help='Set class name of root model', default=None,
 )
@@ -220,16 +246,15 @@ class Config(BaseModel):
             return value  # pragma: no cover
         return Path(value).expanduser().resolve()
 
-    @root_validator
-    def validate_literal_option(cls, values: Dict[str, Any]) -> Dict[str, Any]:
-        if values.get('enum_field_as_literal'):
-            target_python_version: PythonVersion = values['target_python_version']
-            if not target_python_version.has_literal_type:
-                raise Error(
-                    f"`--enum-field-as-literal` isn't compatible with `--target-python-version {target_python_version.value}`.\n"  # type: ignore
-                    f"You have to set `--target-python-version {target_python_version.PY_38.value}` or later version."
-                )
-        return values
+    @validator('url', pre=True)
+    def validate_url(cls, value: Any) -> Optional[ParseResult]:
+        if isinstance(value, str) and is_url(value):  # pragma: no cover
+            return urlparse(value)
+        elif value is None:  # pragma: no cover
+            return None
+        raise Error(
+            f'This protocol doesn\'t support only http/https. --input={value}'
+        )  # pragma: no cover
 
     @root_validator
     def validate_use_generic_container_types(
@@ -244,7 +269,7 @@ class Config(BaseModel):
                 )
         return values
 
-    input: Optional[Path]
+    input: Optional[Union[Path, str]]
     input_file_type: InputFileType = InputFileType.Auto
     output: Optional[Path]
     debug: bool = False
@@ -271,6 +296,10 @@ class Config(BaseModel):
     strict_nullable: bool = False
     use_generic_container_types: bool = False
     enable_faux_immutability: bool = False
+    url: Optional[ParseResult] = None
+    disable_appending_item_suffix: bool = False
+    strict_types: List[StrictTypes] = []
+    empty_enum_field_name: Optional[str] = None
 
     def merge_args(self, args: Namespace) -> None:
         for field_name in self.__fields__:
@@ -362,7 +391,7 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
 
     try:
         generate(
-            input_=config.input or sys.stdin.read(),
+            input_=config.url or config.input or sys.stdin.read(),
             input_file_type=config.input_file_type,
             output=config.output,
             target_python_version=config.target_python_version,
@@ -388,6 +417,9 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
             strict_nullable=config.strict_nullable,
             use_generic_container_types=config.use_generic_container_types,
             enable_faux_immutability=config.enable_faux_immutability,
+            disable_appending_item_suffix=config.disable_appending_item_suffix,
+            strict_types=config.strict_types,
+            empty_enum_field_name=config.empty_enum_field_name,
         )
         return Exit.OK
     except InvalidClassNameError as e:
