@@ -125,13 +125,14 @@ def get_template(template_file_path: Path) -> Template:
 
 
 class TemplateBase(ABC):
-    def __init__(self, template_file_path: Path) -> None:
-        self.template_file_path: Path = template_file_path
-        self._template: Template = get_template(template_file_path)
-
     @property
+    @abstractmethod
+    def template_file_path(self) -> Path:
+        raise NotImplementedError
+
+    @cached_property
     def template(self) -> Template:
-        return self._template
+        return get_template(self.template_file_path)
 
     @abstractmethod
     def render(self) -> str:
@@ -145,8 +146,9 @@ class TemplateBase(ABC):
 
 
 class DataModel(TemplateBase, ABC):
-    TEMPLATE_FILE_PATH: str = ''
-    BASE_CLASS: str = ''
+    TEMPLATE_FILE_PATH: ClassVar[str] = ''
+    BASE_CLASS: ClassVar[str] = ''
+    DEFAULT_IMPORTS: ClassVar[Tuple[Import, ...]] = ()
 
     def __init__(
         self,
@@ -170,6 +172,7 @@ class DataModel(TemplateBase, ABC):
             custom_template_file_path = custom_template_dir / template_file_path.name
             if custom_template_file_path.exists():
                 template_file_path = custom_template_file_path
+        self._template_file_path = template_file_path
 
         self.fields: List[DataModelFieldBase] = fields or []
         self.decorators: List[str] = decorators or []
@@ -207,7 +210,11 @@ class DataModel(TemplateBase, ABC):
         for field in self.fields:
             field.parent = self
 
-        super().__init__(template_file_path=template_file_path)
+        self._additional_imports.extend(self.DEFAULT_IMPORTS)
+
+    @property
+    def template_file_path(self) -> Path:
+        return self._template_file_path
 
     @property
     def imports(self) -> Tuple[Import, ...]:
@@ -235,7 +242,7 @@ class DataModel(TemplateBase, ABC):
             base_class = base_class_full_path.rsplit('.', 1)[-1]
 
         if '.' in self.name:
-            module, class_name = self.name.rsplit('.', 1)
+            module, _ = self.name.rsplit('.', 1)
             prefix = f'{module}.'
             if base_class.startswith(prefix):
                 base_class = base_class.replace(prefix, '', 1)
