@@ -1,6 +1,6 @@
 from collections import ChainMap
 from pathlib import Path
-from typing import Any, DefaultDict, Dict, List, Mapping, Optional, Set, Tuple
+from typing import Any, ClassVar, DefaultDict, Dict, List, Mapping, Optional, Set, Tuple
 
 from pydantic import Field
 
@@ -30,7 +30,13 @@ class Constraints(ConstraintsBase):
 
 
 class DataModelField(DataModelFieldBase):
-    _FIELDS_KEYS: Set[str] = {'alias', 'example', 'examples', 'description', 'title'}
+    _FIELDS_KEYS: ClassVar[Set[str]] = {
+        'alias',
+        'example',
+        'examples',
+        'description',
+        'title',
+    }
 
     constraints: Optional[Constraints] = None
 
@@ -57,9 +63,18 @@ class DataModelField(DataModelFieldBase):
 
         return result
 
+    def self_reference(self) -> bool:
+        return isinstance(self.parent, BaseModel) and self.parent.reference.path in {
+            d.reference.path for d in self.data_type.all_data_types if d.reference
+        }
+
     def __str__(self) -> str:
         data: Mapping[str, Any] = self.dict(include=self._FIELDS_KEYS)
-        if self.constraints is not None:
+        if (
+            self.constraints is not None
+            and not self.self_reference()
+            and not self.data_type.strict
+        ):
             data = ChainMap(data, self.constraints.dict())
         field_arguments = sorted(
             f"{k}={repr(v)}" for k, v in data.items() if v is not None
@@ -75,8 +90,8 @@ class DataModelField(DataModelFieldBase):
 
 
 class BaseModel(DataModel):
-    TEMPLATE_FILE_PATH = 'pydantic/BaseModel.jinja2'
-    BASE_CLASS = 'pydantic.BaseModel'
+    TEMPLATE_FILE_PATH: ClassVar[str] = 'pydantic/BaseModel.jinja2'
+    BASE_CLASS: ClassVar[str] = 'pydantic.BaseModel'
 
     def __init__(
         self,
