@@ -14,11 +14,14 @@ from typing import (
     List,
     Mapping,
     Optional,
+    Pattern,
+    Sequence,
     Set,
     Tuple,
     Type,
     Union,
 )
+from urllib.parse import ParseResult
 
 from pydantic import BaseModel
 
@@ -30,11 +33,23 @@ from ..model import pydantic as pydantic_model
 from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase
 from ..model.enum import Enum
 from ..reference import ModelResolver, Reference
-from ..types import DataType, DataTypeManager
-from . import LiteralType
+from ..types import DataType, DataTypeManager, StrictTypes
+from . import DefaultPutDict, LiteralType
 
-_UNDER_SCORE_1 = re.compile(r'(.)([A-Z][a-z]+)')
-_UNDER_SCORE_2 = re.compile('([a-z0-9])([A-Z])')
+_UNDER_SCORE_1: Pattern[str] = re.compile(r'(.)([A-Z][a-z]+)')
+_UNDER_SCORE_2: Pattern[str] = re.compile('([a-z0-9])([A-Z])')
+
+escape_characters = str.maketrans(
+    {
+        "\\": r"\\",
+        "'": r"\'",
+        '\b': r'\b',
+        '\f': r'\f',
+        '\n': r'\n',
+        '\r': r'\r',
+        '\t': r'\t',
+    }
+)
 
 
 @lru_cache()
@@ -154,7 +169,7 @@ def sort_data_models(
                 continue
             # unresolved
             unresolved_classes = ', '.join(
-                f"[class: {item.name} references: {item.reference_classes}]"
+                f"[class: {item.path} references: {item.reference_classes}]"
                 for item in unresolved_references
             )
             raise Exception(f'A Parser can not resolve classes: {unresolved_classes}.')
@@ -216,7 +231,7 @@ class Source(BaseModel):
 class Parser(ABC):
     def __init__(
         self,
-        source: Union[str, Path, List[Path]],
+        source: Union[str, Path, List[Path], ParseResult],
         *,
         data_model_type: Type[DataModel] = pydantic_model.BaseModel,
         data_model_root_type: Type[DataModel] = pydantic_model.CustomRootType,
@@ -246,9 +261,16 @@ class Parser(ABC):
         strict_nullable: bool = False,
         use_generic_container_types: bool = False,
         enable_faux_immutability: bool = False,
+        remote_text_cache: Optional[DefaultPutDict[str, str]] = None,
+        disable_appending_item_suffix: bool = False,
+        strict_types: Optional[Sequence[StrictTypes]] = None,
+        empty_enum_field_name: Optional[str] = None,
     ):
         self.data_type_manager: DataTypeManager = data_type_manager_type(
-            target_python_version, use_standard_collections, use_generic_container_types
+            target_python_version,
+            use_standard_collections,
+            use_generic_container_types,
+            strict_types,
         )
         self.data_model_type: Type[DataModel] = data_model_type
         self.data_model_root_type: Type[DataModel] = data_model_root_type
@@ -264,8 +286,12 @@ class Parser(ABC):
         self.field_constraints: bool = field_constraints
         self.snake_case_field: bool = snake_case_field
         self.strip_default_none: bool = strip_default_none
-        self.apply_default_values_for_required_fields: bool = apply_default_values_for_required_fields
-        self.force_optional_for_required_fields: bool = force_optional_for_required_fields
+        self.apply_default_values_for_required_fields: bool = (
+            apply_default_values_for_required_fields
+        )
+        self.force_optional_for_required_fields: bool = (
+            force_optional_for_required_fields
+        )
         self.use_schema_description: bool = use_schema_description
         self.reuse_model: bool = reuse_model
         self.encoding: str = encoding
@@ -275,6 +301,9 @@ class Parser(ABC):
         self.use_generic_container_types: bool = use_generic_container_types
         self.enable_faux_immutability: bool = enable_faux_immutability
 
+        self.remote_text_cache: DefaultPutDict[str, str] = (
+            remote_text_cache or DefaultPutDict()
+        )
         self.current_source_path: Optional[Path] = None
 
         if base_path:
@@ -286,7 +315,7 @@ class Parser(ABC):
         else:
             self.base_path = Path.cwd()
 
-        self.source: Union[str, Path, List[Path]] = source
+        self.source: Union[str, Path, List[Path], ParseResult] = source
         self.custom_template_dir = custom_template_dir
         self.extra_template_data: DefaultDict[
             str, Any
@@ -298,7 +327,12 @@ class Parser(ABC):
         if enable_faux_immutability:
             self.extra_template_data[ALL_MODEL]['allow_mutation'] = False
 
-        self.model_resolver = ModelResolver(aliases=aliases)
+        self.model_resolver = ModelResolver(
+            aliases=aliases,
+            base_url=source.geturl() if isinstance(source, ParseResult) else None,
+            singular_name_suffix='' if disable_appending_item_suffix else None,
+            empty_field_name=empty_enum_field_name,
+        )
         self.field_preprocessors: List[
             Callable[[DataModelFieldBase, DataModel], None]
         ] = []
@@ -322,6 +356,25 @@ class Parser(ABC):
         elif isinstance(self.source, list):  # pragma: no cover
             for path in self.source:
                 yield Source.from_path(path, self.base_path, self.encoding)
+        else:
+            yield Source(
+                path=Path(self.source.path),
+                text=self.remote_text_cache.get_or_put(
+                    self.source.geturl(), default_factory=self._get_text_from_url
+                ),
+            )
+
+    def _get_text_from_url(self, url: str) -> str:
+        from ..http import get_body
+
+        return self.remote_text_cache.get_or_put(url, default_factory=get_body)
+
+    @classmethod
+    def get_url_path_parts(cls, url: ParseResult) -> List[str]:
+        return [
+            f'{url.scheme}://{url.hostname}',
+            *url.path.split('/')[1:],
+        ]
 
     def append_result(self, data_model: DataModel) -> None:
         for field_preprocessor in self.field_preprocessors:
@@ -406,7 +459,9 @@ class Parser(ABC):
                         model.reference.name = generated_name
 
         for module, models in module_models:
-            module_path = '.'.join(module)
+            for model in models:
+                model.reference.module_name = '.'.join(module)
+        for module, models in module_models:
 
             init = False
             if module:
@@ -424,10 +479,8 @@ class Parser(ABC):
             result: List[str] = []
             imports = Imports()
             scoped_model_resolver = ModelResolver()
-            import_map: Dict[str, Tuple[str, str]] = {}
 
             for model in models:
-                alias_map: Dict[str, Optional[str]] = {}
                 imports.append(model.imports)
                 for data_type in model.all_data_types:
                     # To change from/import
@@ -437,51 +490,19 @@ class Parser(ABC):
                         # Or, Referenced model is in the same file. we don't need to import the model
                         continue
 
-                    elif (
-                        isinstance(self.source, Path)
-                        and self.source.is_file()
-                        and self.source.name == data_type.reference.path.split('#/')[0]
-                    ):
-                        # The input file is no need to import
-                        full_name = data_type.reference.name
-                    else:
-                        full_name = data_type.full_name or data_type.reference.name
-                    from_, import_ = full_path = relative(module_path, full_name)
+                    from_, import_ = full_path = relative(
+                        model.reference.module_name, data_type.full_name
+                    )
 
-                    alias = scoped_model_resolver.add(
-                        full_path, import_, unique=True
-                    ).name
-                    if alias != import_:
-                        alias_map[data_type.reference.path] = alias
+                    alias = scoped_model_resolver.add(full_path, import_).name
 
                     name = data_type.reference.short_name
-                    data_type.alias = (
-                        f'{alias}.{name}'
-                        if (from_ and import_ and alias != name)
-                        else name
-                    )
+                    if from_ and import_ and alias != name:
+                        data_type.alias = f'{alias}.{name}'
 
-                    if (
-                        full_name
-                        and full_name.startswith(from_)
-                        or f'.{full_name}'.startswith(from_)
-                    ):
-                        import_map[data_type.reference.path] = full_path
-                for ref_path in model.reference_classes:
-                    reference: Reference = self.model_resolver.get(ref_path)  # type: ignore
-                    if reference.source in models:
-                        continue
-                    elif ref_path in import_map:
-                        from_, import_ = import_map[ref_path]
-                    else:
-                        from_, import_ = relative(module_path, reference.name)
                     if init:
                         from_ += "."
-                    imports.append(
-                        Import(
-                            from_=from_, import_=import_, alias=alias_map.get(ref_path),
-                        )
-                    )
+                    imports.append(Import(from_=from_, import_=import_, alias=alias))
 
             if self.reuse_model:
                 model_cache: Dict[Tuple[str, ...], Reference] = {}
