@@ -13,11 +13,13 @@ from typing import (
     List,
     Mapping,
     Optional,
+    Sequence,
     Set,
     Tuple,
     Type,
     Union,
 )
+from urllib.parse import ParseResult
 from warnings import warn
 
 from pydantic import BaseModel, Field, root_validator, validator
@@ -26,17 +28,18 @@ from datamodel_code_generator import (
     InvalidClassNameError,
     cached_property,
     load_yaml,
+    load_yaml_from_path,
     snooper_to_methods,
 )
 from datamodel_code_generator.format import PythonVersion
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model.enum import Enum
-from datamodel_code_generator.parser import LiteralType
+from datamodel_code_generator.parser import DefaultPutDict, LiteralType
 
 from ..model import pydantic as pydantic_model
-from ..parser.base import Parser
+from ..parser.base import Parser, escape_characters
 from ..reference import Reference, is_url
-from ..types import DataType, DataTypeManager, Types
+from ..types import DataType, DataTypeManager, StrictTypes, Types
 
 
 def get_model_by_path(schema: Dict[str, Any], keys: List[str]) -> Dict[str, Any]:
@@ -225,7 +228,7 @@ JsonSchemaObject.update_forward_refs()
 class JsonSchemaParser(Parser):
     def __init__(
         self,
-        source: Union[str, Path, List[Path]],
+        source: Union[str, Path, List[Path], ParseResult],
         *,
         data_model_type: Type[DataModel] = pydantic_model.BaseModel,
         data_model_root_type: Type[DataModel] = pydantic_model.CustomRootType,
@@ -255,6 +258,10 @@ class JsonSchemaParser(Parser):
         strict_nullable: bool = False,
         use_generic_container_types: bool = False,
         enable_faux_immutability: bool = False,
+        remote_text_cache: Optional[DefaultPutDict[str, str]] = None,
+        disable_appending_item_suffix: bool = False,
+        strict_types: Optional[Sequence[StrictTypes]] = None,
+        empty_enum_field_name: Optional[str] = None,
     ):
         super().__init__(
             source=source,
@@ -286,9 +293,13 @@ class JsonSchemaParser(Parser):
             strict_nullable=strict_nullable,
             use_generic_container_types=use_generic_container_types,
             enable_faux_immutability=enable_faux_immutability,
+            remote_text_cache=remote_text_cache,
+            disable_appending_item_suffix=disable_appending_item_suffix,
+            strict_types=strict_types,
+            empty_enum_field_name=empty_enum_field_name,
         )
 
-        self.remote_object_cache: Dict[str, Dict[str, Any]] = {}
+        self.remote_object_cache: DefaultPutDict[str, Dict[str, Any]] = DefaultPutDict()
         self.raw_obj: Dict[Any, Any] = {}
         self._root_id: Optional[str] = None
         self._root_id_base_path: Optional[str] = None
@@ -424,9 +435,7 @@ class JsonSchemaParser(Parser):
         # ignore an undetected object
         if ignore_duplicate_model and not fields and len(base_classes) == 1:
             return self.data_type(reference=base_classes[0])
-        reference = self.model_resolver.add(
-            path, name, class_name=True, unique=True, loaded=True
-        )
+        reference = self.model_resolver.add(path, name, class_name=True, loaded=True)
         self.set_additional_properties(reference.name, obj)
         data_model_type = self.data_model_type(
             reference=reference,
@@ -513,7 +522,7 @@ class JsonSchemaParser(Parser):
                             field.additionalProperties
                         ):
                             additional_properties_type = self.data_type(
-                                literals=field.additionalProperties.enum
+                                literals=field.additionalProperties.enum,
                             )
                         else:
                             additional_properties_type = self.parse_enum(
@@ -607,12 +616,7 @@ class JsonSchemaParser(Parser):
                 f'This argument will be removed in a future version'
             )
         reference = self.model_resolver.add(
-            path,
-            name,
-            class_name=True,
-            singular_name=singular_name,
-            unique=True,
-            loaded=True,
+            path, name, class_name=True, singular_name=singular_name, loaded=True,
         )
         class_name = reference.name
         self.set_title(class_name, obj)
@@ -637,11 +641,7 @@ class JsonSchemaParser(Parser):
             if item.has_constraint and (obj.has_constraint or self.field_constraints):
                 return self.parse_root_type(
                     self.model_resolver.add(
-                        field_path,
-                        name,
-                        class_name=True,
-                        singular_name=True,
-                        unique=True,
+                        field_path, name, class_name=True, singular_name=True,
                     ).name,
                     item,
                     field_path,
@@ -793,15 +793,24 @@ class JsonSchemaParser(Parser):
         if None in obj.enum and obj.type == 'string':
             # Nullable is valid in only OpenAPI
             nullable: bool = True
-            enum_times = [e for e in obj.enum if e]
+            enum_times = [e for e in obj.enum if e is not None]
         else:
             enum_times = obj.enum
             nullable = False
 
+        exclude_field_names: Set[str] = set()
+
         for i, enum_part in enumerate(enum_times):
             if obj.type == 'string' or isinstance(enum_part, str):
-                default = f"'{enum_part}'"
-                field_name = enum_part
+                default = (
+                    f"'{enum_part.translate(escape_characters)}'"
+                    if isinstance(enum_part, str)
+                    else enum_part
+                )
+                if obj.x_enum_varnames:
+                    field_name = obj.x_enum_varnames[i]
+                else:
+                    field_name = str(enum_part)
             else:
                 default = enum_part
                 if obj.x_enum_varnames:
@@ -813,9 +822,13 @@ class JsonSchemaParser(Parser):
                         else type(enum_part).__name__
                     )
                     field_name = f'{prefix}_{enum_part}'
+            field_name = self.model_resolver.get_valid_name(
+                field_name, excludes=exclude_field_names
+            )
+            exclude_field_names.add(field_name)
             enum_fields.append(
                 self.data_model_field_type(
-                    name=self.model_resolver.get_valid_name(field_name),
+                    name=field_name,
                     default=default,
                     data_type=self.data_type_manager.get_data_type(Types.any),
                     required=True,
@@ -829,7 +842,6 @@ class JsonSchemaParser(Parser):
                 class_name=True,
                 singular_name=singular_name,
                 singular_name_suffix='Enum',
-                unique=True,
                 loaded=True,
             )
             enum = Enum(
@@ -847,7 +859,6 @@ class JsonSchemaParser(Parser):
             class_name=True,
             singular_name=singular_name,
             singular_name_suffix='Enum',
-            unique=True,
             loaded=True,
         )
         enum_reference = self.model_resolver.add(
@@ -856,7 +867,6 @@ class JsonSchemaParser(Parser):
             class_name=True,
             singular_name=singular_name,
             singular_name_suffix='Enum',
-            unique=True,
             loaded=True,
         )
         enum = Enum(
@@ -893,44 +903,20 @@ class JsonSchemaParser(Parser):
         return self._get_ref_body_from_remote(resolved_ref)
 
     def _get_ref_body_from_url(self, ref: str) -> Dict[Any, Any]:
-        if ref[-1] == '#':
-            ref = ref[:-1]
-        cached_ref_body: Optional[Dict[str, Any]] = self.remote_object_cache.get(ref)
-        if cached_ref_body:
-            return cached_ref_body
-
         # URL Reference – $ref: 'http://path/to/your/resource' Uses the whole document located on the different server.
-        try:
-            import httpx
-        except ImportError:  # pragma: no cover
-            raise Exception(
-                f'Please run $pip install datamodel-code-generator[http] to resolve URL Reference ref={ref}'
-            )
-        raw_body: str = httpx.get(ref).text
-        # yaml loader can parse json data.
-        ref_body = load_yaml(raw_body)
-        self.remote_object_cache[ref] = ref_body
-        return ref_body
+        return self.remote_object_cache.get_or_put(
+            ref, default_factory=lambda key: load_yaml(self._get_text_from_url(key))
+        )
 
     def _get_ref_body_from_remote(self, resolved_ref: str) -> Dict[Any, Any]:
         # Remote Reference – $ref: 'document.json' Uses the whole document located on the same server and in
         # the same location. TODO treat edge case
-        if resolved_ref[-1] == '#':
-            resolved_ref = resolved_ref[:-1]
         full_path = self.base_path / resolved_ref
-        ref_full_path = str(full_path)
 
-        cached_ref_body: Optional[Dict[str, Any]] = self.remote_object_cache.get(
-            ref_full_path
+        return self.remote_object_cache.get_or_put(
+            str(full_path),
+            default_factory=lambda _: load_yaml_from_path(full_path, self.encoding),
         )
-        if cached_ref_body:
-            return cached_ref_body
-
-        # yaml loader can parse json data.
-        with full_path.open(encoding=self.encoding) as f:
-            ref_body = load_yaml(f)
-        self.remote_object_cache[ref_full_path] = ref_body
-        return ref_body
 
     def parse_ref(self, obj: JsonSchemaObject, path: List[str]) -> None:
         if obj.ref:
@@ -944,14 +930,6 @@ class JsonSchemaParser(Parser):
                 elif self.model_resolver.is_after_load(obj.ref):
                     self.reserved_refs[tuple(ref.split('#')[0].split('/'))].add(ref)  # type: ignore
                 else:
-                    if self.model_resolver.is_external_root_ref(ref):
-                        relative_path, object_path = ref[:-1], ''
-                    else:
-                        relative_path, object_path = ref.split('#')
-
-                    models = self._get_ref_body(relative_path)
-                    model_name = self.model_resolver.add_ref(obj.ref).name
-
                     if is_url(ref):
                         relative_path, object_path = ref.split('#')
                         relative_paths = [relative_path]
@@ -961,12 +939,14 @@ class JsonSchemaParser(Parser):
                         else:
                             relative_path, object_path = ref.split('#')
                         relative_paths = relative_path.split('/')
-                    self._parse_file(
-                        self._get_ref_body(relative_path),
-                        model_name,
-                        relative_paths,
-                        object_path.split('/') if object_path else None,
-                    )
+
+                    with self.model_resolver.base_url_context(relative_path):
+                        self._parse_file(
+                            self._get_ref_body(relative_path),
+                            self.model_resolver.add_ref(obj.ref).name,
+                            relative_paths,
+                            object_path.split('/') if object_path else None,
+                        )
                     self.model_resolver.add_ref(obj.ref,).loaded = True
 
         if obj.items:
@@ -1012,7 +992,7 @@ class JsonSchemaParser(Parser):
         previous_root_id: Optional[str] = self.root_id
         if root_id:
             try:
-                resolved_ref = self.model_resolver.resolve_ref(root_id)
+                resolved_ref = self.model_resolver.resolve_ref(root_id).split('#')[0]
                 self._get_ref_body(resolved_ref)
             except Exception as e:
                 print(f'Parse $id failed. $id={root_id}\n {str(e)}', file=sys.stderr)
@@ -1051,7 +1031,10 @@ class JsonSchemaParser(Parser):
             }
 
         for source in self.iter_source:
-            path_parts = list(source.path.parts)
+            if isinstance(self.source, ParseResult):
+                path_parts = self.get_url_path_parts(self.source)
+            else:
+                path_parts = list(source.path.parts)
             if self.current_source_path is not None:
                 self.current_source_path = source.path
             with self.model_resolver.current_root_context(path_parts):
