@@ -7,6 +7,7 @@ from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
     ClassVar,
     DefaultDict,
     Dict,
@@ -18,6 +19,7 @@ from typing import (
     Sequence,
     Set,
     Tuple,
+    TypeVar,
     Union,
 )
 
@@ -69,6 +71,7 @@ class Reference(_BaseModel):
     loaded: bool = True
     source: Optional[Any] = None
     children: List[Any] = []
+    module_name: str = ''
     _exclude_fields: ClassVar = {'children'}
 
     @validator('original_name')
@@ -84,28 +87,29 @@ class Reference(_BaseModel):
         arbitrary_types_allowed = True
         keep_untouched = (cached_property,)
 
-    @property
-    def module_name(self) -> Optional[str]:
-        if is_url(self.path):  # pragma: no cover
-            return None
-        # TODO: Support file:///
-        path = Path(self.path.split('#')[0])
-
-        # workaround: If a file name has dot then, this method uses first part.
-        module_name = f'{".".join(path.parts[:-1])}.{path.stem.split(".")[0]}'
-        if module_name.startswith(f'.{self.name.split(".", 1)[0]}'):
-            return None
-        elif module_name == '.':  # pragma: no cover
-            return None
-        return module_name
-
     @property
     def short_name(self) -> str:
         return self.name.rsplit('.', 1)[-1]
 
 
+SINGULAR_NAME_SUFFIX: str = 'Item'
+
 ID_PATTERN: Pattern[str] = re.compile(r'^#[^/].*')
 
+T = TypeVar('T')
+
+
+@contextmanager
+def context_variable(
+    setter: Callable[[T], None], current_value: T, new_value: T
+) -> Generator[None, None, None]:
+    previous_value: T = current_value
+    setter(new_value)
+    try:
+        yield
+    finally:
+        setter(previous_value)
+
 
 class ModelResolver:
     def __init__(
@@ -113,6 +117,9 @@ class ModelResolver:
         aliases: Optional[Mapping[str, str]] = None,
         exclude_names: Set[str] = None,
         duplicate_name_suffix: Optional[str] = None,
+        base_url: Optional[str] = None,
+        singular_name_suffix: Optional[str] = None,
+        empty_field_name: Optional[str] = None,
     ) -> None:
         self.references: Dict[str, Reference] = {}
         self.aliases: Mapping[str, str] = {} if aliases is None else {**aliases}
@@ -122,6 +129,27 @@ class ModelResolver:
         self.after_load_files: Set[str] = set()
         self.exclude_names: Set[str] = exclude_names or set()
         self.duplicate_name_suffix: Optional[str] = duplicate_name_suffix
+        self._base_url: Optional[str] = base_url
+        self.singular_name_suffix: str = singular_name_suffix if isinstance(
+            singular_name_suffix, str
+        ) else SINGULAR_NAME_SUFFIX
+        self.empty_field_name: str = empty_field_name or '_'
+
+    @property
+    def base_url(self) -> Optional[str]:
+        return self._base_url
+
+    def set_base_url(self, base_url: Optional[str]) -> None:
+        self._base_url = base_url
+
+    @contextmanager
+    def base_url_context(self, base_url: str) -> Generator[None, None, None]:
+
+        if self._base_url:
+            with context_variable(self.set_base_url, self.base_url, base_url):
+                yield
+        else:
+            yield
 
     @property
     def current_root(self) -> Sequence[str]:
@@ -136,10 +164,8 @@ class ModelResolver:
     def current_root_context(
         self, current_root: Sequence[str]
     ) -> Generator[None, None, None]:
-        previous_root_path: Sequence[str] = self.current_root
-        self.set_current_root(current_root)
-        yield
-        self.set_current_root(previous_root_path)
+        with context_variable(self.set_current_root, self.current_root, current_root):
+            yield
 
     @property
     def root_id_base_path(self) -> Optional[str]:
@@ -157,20 +183,28 @@ class ModelResolver:
         else:
             joined_path = self.join_path(path)
         if ID_PATTERN.match(joined_path):
-            return self.ids['/'.join(self.current_root)][joined_path]
+            ref: str = self.ids['/'.join(self.current_root)][joined_path]
         elif '#' in joined_path:
             if joined_path[0] == '#':
                 joined_path = f'{"/".join(self.current_root)}{joined_path}'
             if self.is_remote_ref(joined_path):
-                return f'{"/".join(self.current_root[:-1])}/{joined_path}'
-            delimiter = joined_path.index('#')
-            return f"{''.join(joined_path[:delimiter])}#{''.join(joined_path[delimiter + 1:])}"
+                ref = f'{"/".join(self.current_root[:-1])}/{joined_path}'
+            else:
+                delimiter = joined_path.index('#')
+                ref = f"{''.join(joined_path[:delimiter])}#{''.join(joined_path[delimiter + 1:])}"
         elif self.root_id_base_path and self.current_root != path:
-            return f'{self.root_id_base_path}/{joined_path}#'
-        joined_path = f'{joined_path}#'
-        if self.is_remote_ref(joined_path):
-            return f'{"/".join(self.current_root[:-1])}/{joined_path}'
-        return joined_path
+            ref = f'{self.root_id_base_path}/{joined_path}#'
+        else:
+            joined_path = f'{joined_path}#'
+            if self.is_remote_ref(joined_path):
+                ref = f'{"/".join(self.current_root[:-1])}/{joined_path}'
+            else:
+                ref = joined_path
+        if self.base_url:
+            from .http import join_url
+
+            return join_url(self.base_url, ref)
+        return ref
 
     def is_remote_ref(self, resolved_ref: str) -> bool:
         return (
@@ -236,8 +270,8 @@ class ModelResolver:
         *,
         class_name: bool = False,
         singular_name: bool = False,
-        unique: bool = False,
-        singular_name_suffix: str = 'Item',
+        unique: bool = True,
+        singular_name_suffix: Optional[str] = None,
         loaded: bool = False,
     ) -> Reference:
         joined_path = self.join_path(path)
@@ -253,9 +287,13 @@ class ModelResolver:
                 return reference
         name = original_name
         if singular_name:
-            name = get_singular_name(name, singular_name_suffix)
+            name = get_singular_name(
+                name, singular_name_suffix or self.singular_name_suffix
+            )
         if class_name:
-            name = self.get_class_name(name, unique)
+            name = self.get_class_name(
+                name, unique, reference.name if reference else None
+            )
         elif unique:
             name = self._get_uniq_name(name)
         if reference:
@@ -272,7 +310,9 @@ class ModelResolver:
     def get(self, path: Union[Sequence[str], str]) -> Optional[Reference]:
         return self.references.get(self.resolve_ref(path))
 
-    def get_class_name(self, field_name: str, unique: bool = True) -> str:
+    def get_class_name(
+        self, field_name: str, unique: bool = True, reserved_name: Optional[str] = None
+    ) -> str:
         if '.' in field_name:
             split_name = [self.get_valid_name(n) for n in field_name.split('.')]
             prefix, field_name = '.'.join(split_name[:-1]), split_name[-1]
@@ -283,6 +323,8 @@ class ModelResolver:
         field_name = self.get_valid_name(field_name)
         upper_camel_name = snake_to_upper_camel(field_name)
         if unique:
+            if reserved_name == upper_camel_name:
+                return upper_camel_name
             class_name = self._get_uniq_name(upper_camel_name, camel=True)
         else:
             class_name = upper_camel_name
@@ -313,19 +355,27 @@ class ModelResolver:
     def validate_name(cls, name: str) -> bool:
         return name.isidentifier() and not iskeyword(name)
 
-    @lru_cache()
-    def get_valid_name(self, name: str, camel: bool = False) -> str:
-        if name.isidentifier():
-            return name
+    def get_valid_name(
+        self, name: str, camel: bool = False, excludes: Optional[Set[str]] = None
+    ) -> str:
+        if not name:
+            name = self.empty_field_name
         if name[0] == '#':
             name = name[1:]
         # TODO: when first character is a number
-        replaced_name = re.sub(r'\W', '_', name)
-        if replaced_name[0].isnumeric():
-            replaced_name = f'field_{replaced_name}'
-        # if replaced_name.isidentifier() and not iskeyword(replaced_name):
-        # return self.get_uniq_name(replaced_name, camel)
-        return replaced_name
+        name = re.sub(r'\W', '_', name)
+        if name[0].isnumeric():
+            name = f'field_{name}'
+        count = 1
+        new_name = name
+        while (
+            not new_name.isidentifier()
+            and iskeyword(new_name)
+            or (excludes and new_name in excludes)
+        ):
+            new_name = f'{name}_{count}'
+            count += 1
+        return new_name
 
     def get_valid_field_name_and_alias(
         self, field_name: str
@@ -337,7 +387,7 @@ class ModelResolver:
 
 
 @lru_cache()
-def get_singular_name(name: str, suffix: str = 'Item') -> str:
+def get_singular_name(name: str, suffix: str = SINGULAR_NAME_SUFFIX) -> str:
     singular_name = inflect_engine.singular_noun(name)
     if singular_name is False:
         singular_name = f'{name}{suffix}'
