@@ -2,6 +2,7 @@ from abc import ABC, abstractmethod
 from enum import Enum, auto
 from itertools import chain
 from typing import (
+    TYPE_CHECKING,
     Any,
     ClassVar,
     Dict,
@@ -10,12 +11,15 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Sequence,
     Set,
     Tuple,
     Type,
     TypeVar,
 )
 
+from pydantic import create_model
+
 from datamodel_code_generator.format import PythonVersion
 from datamodel_code_generator.imports import (
     IMPORT_ABC_MAPPING,
@@ -23,6 +27,7 @@ from datamodel_code_generator.imports import (
     IMPORT_DICT,
     IMPORT_LIST,
     IMPORT_LITERAL,
+    IMPORT_LITERAL_BACKPORT,
     IMPORT_MAPPING,
     IMPORT_OPTIONAL,
     IMPORT_SEQUENCE,
@@ -34,6 +39,14 @@ from datamodel_code_generator.reference import Reference, _BaseModel
 T = TypeVar('T')
 
 
+class StrictTypes(Enum):
+    str = 'str'
+    bytes = 'bytes'
+    int = 'int'
+    float = 'float'
+    bool = 'bool'
+
+
 def chain_as_tuple(*iterables: Iterable[T]) -> Tuple[T, ...]:
     return tuple(chain(*iterables))
 
@@ -55,10 +68,33 @@ class DataType(_BaseModel):
     alias: Optional[str] = None
     parent: Optional[Any] = None
     children: List[Any] = []
+    strict: bool = False
 
     _exclude_fields: ClassVar[Set[str]] = {'parent', 'children'}
     _pass_fields: ClassVar[Set[str]] = {'parent', 'children', 'data_types', 'reference'}
 
+    @classmethod
+    def from_import(
+        cls,
+        import_: Import,
+        *,
+        is_optional: bool = False,
+        is_dict: bool = False,
+        is_list: bool = False,
+        strict: bool = False,
+        kwargs: Optional[Dict[str, Any]] = None,
+    ) -> 'DataType':
+        return cls(
+            type=import_.import_,
+            imports=[import_],
+            is_optional=is_optional,
+            is_dict=is_dict,
+            is_list=is_list,
+            is_func=True if kwargs else False,
+            strict=strict,
+            kwargs=kwargs,
+        )
+
     @property
     def unresolved_types(self) -> FrozenSet[str]:
         return frozenset(
@@ -90,8 +126,8 @@ class DataType(_BaseModel):
     def full_name(self) -> str:
         module_name = self.module_name
         if module_name:
-            return f'{module_name}.{self.type}'
-        return self.type  # type: ignore
+            return f'{module_name}.{self.reference.short_name}'  # type: ignore
+        return self.reference.short_name  # type: ignore
 
     @property
     def all_data_types(self) -> Iterator['DataType']:
@@ -124,8 +160,18 @@ class DataType(_BaseModel):
         imports: Tuple[Tuple[bool, Import], ...] = (
             (self.is_optional, IMPORT_OPTIONAL),
             (len(self.data_types) > 1, IMPORT_UNION),
-            (any(self.literals), IMPORT_LITERAL),
         )
+        if any(self.literals):
+            import_literal = (
+                IMPORT_LITERAL
+                if self.python_version.has_literal_type
+                else IMPORT_LITERAL_BACKPORT
+            )
+            imports = (
+                *imports,
+                (any(self.literals), import_literal),
+            )
+
         if self.use_generic_container:
             if self.use_standard_collections:
                 imports = (
@@ -206,23 +252,6 @@ class DataType(_BaseModel):
 DataType.update_forward_refs()
 
 
-class DataTypePy36(DataType):
-    python_version: PythonVersion = PythonVersion.PY_36
-
-
-class DataTypeStandardCollections(DataType):
-    use_standard_collections: bool = True
-
-
-class DataTypeGenericContainer(DataType):
-    use_generic_container: bool = True
-
-
-class DataTypeGenericContainerStandardCollections(DataType):
-    use_standard_collections: bool = True
-    use_generic_container: bool = True
-
-
 class Types(Enum):
     integer = auto()
     int32 = auto()
@@ -262,28 +291,31 @@ class DataTypeManager(ABC):
         python_version: PythonVersion = PythonVersion.PY_37,
         use_standard_collections: bool = False,
         use_generic_container_types: bool = False,
+        strict_types: Optional[Sequence[StrictTypes]] = None,
     ) -> None:
         self.python_version = python_version
         self.use_standard_collections: bool = use_standard_collections
         self.use_generic_container_types: bool = use_generic_container_types
+        self.strict_types: Sequence[StrictTypes] = strict_types or ()
 
-        self.data_type: Type[DataType]
-        if use_generic_container_types:
-            if python_version == PythonVersion.PY_36:  # pragma: no cover
-                raise Exception(
-                    "use_generic_container_types can not be used with target_python_version 3.6.\n"
-                    " The verison will be not supported in a future version"
-                )
-            if use_standard_collections:
-                self.data_type = DataTypeGenericContainerStandardCollections
-            else:
-                self.data_type = DataTypeGenericContainer
-        elif use_standard_collections:
-            self.data_type = DataTypeStandardCollections
-        elif python_version == PythonVersion.PY_36:
-            self.data_type = DataTypePy36
+        if (
+            use_generic_container_types and python_version == PythonVersion.PY_36
+        ):  # pragma: no cover
+            raise Exception(
+                "use_generic_container_types can not be used with target_python_version 3.6.\n"
+                " The version will be not supported in a future version"
+            )
+
+        if TYPE_CHECKING:
+            self.data_type: Type[DataType]
         else:
-            self.data_type = DataType
+            self.data_type: Type[DataType] = create_model(
+                'ContextDataType',
+                python_version=python_version,
+                use_standard_collections=use_standard_collections,
+                use_generic_container=use_generic_container_types,
+                __base__=DataType,
+            )
 
     @abstractmethod
     def get_data_type(self, types: Types, **kwargs: Any) -> DataType:
