@@ -1,6 +1,6 @@
 import pytest
 
-from datamodel_code_generator import DataTypeManager
+from datamodel_code_generator.model.pydantic import DataTypeManager
 from datamodel_code_generator.model.pydantic.base_model import BaseModel, DataModelField
 from datamodel_code_generator.reference import Reference
 from datamodel_code_generator.types import DataType, Types
